#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)

import os
import json
from datetime import datetime, date, time, timedelta

# Local imports
from utils.utils_grib import *

"""
Given a dataset/variable a.k.a QAR
Return list of mandatory dates to find
"""
def get_forms_dates(dataset, variable, origin, system, forms_dir):
    constraints = {}
    
    # Load constraints file into dictionary
    try:            
        with open(os.path.join(forms_dir, dataset + '/constraints.json'), 'r') as cdf:
            constraints = json.load(cdf)                
    except:
        raise Exception('Problem loading constraints from cds-forms directory. Please check your configuration')

    # In particular dates for a given (dataset, variable)
    md = []
    try:
        dates = []
        for c in constraints:

            # Filter by variable
            if variable in c['variable']:

                # Seasonal needs filter by origin/system
                if 'seasonal' in dataset:
                    if not(origin in c['originating_centre']) or not(system in c['system']):
                        continue

                # Parse all dates
                for y in c['year']:
                    for m in c['month']:
                        if 'day' in c:
                            for d in c['day']:
                                dates.append(datetime(int(y), int(m), int(d)))
                        else:
                            dates.append(datetime(int(y), int(m), int(1)))

        # Get unique ordered values
        md = sorted(list(set(dates)))
    except Exception as e:
        print(e)
        raise Exception('Problem loading constraints from cds-forms directory. Please check your dataset name')

    return md

"""
Given a cds variable name (long-name)
Return a parameter identifier in the mapping.json
"""
def get_param_id(dataset, variable, forms_dir):
    mapping = {}
    code = None

    # Load constraints file into dictionary
    try:            
        with open(os.path.join(forms_dir, dataset + '/mapping.json'), 'r') as cdf:
            mapping = json.load(cdf)                
    except:
        raise Exception('Problem loading mapping from cds-forms directory. Please check your configuration')
 
    # Fetch given variable code
    try:            
        code = mapping['remap']['variable'][variable]              
    except:
        raise Exception('Could not find variable {} in dataset {}'.format(variable, dataset))


    return code

"""
Given a combination of dataset/variable
Return GRIB metadata available
"""
def get_metadata(forms_dir, dataset, variable):

    grib_info = load_grib_metadata()
    paramId = get_param_id(dataset, variable, forms_dir)

    return grib_info[str(paramId)]